@@ -10,7 +10,7 @@ It is strongly recommended to run these tests via Docker, to ensure you have
 all dependencies correctly installed. See ../docs/.
 """
 
-# pylint: disable=missing-docstring
+# pylint: disable=missing-class-docstring,missing-function-docstring
 
 import argparse
 import collections
@@ -21,7 +21,7 @@ import glob
 import inspect
 import os
 import sys
-import multiprocessing
+import pdb
 import pstats
 import random
 import re
@@ -30,17 +30,18 @@ import subprocess
 import tempfile
 import threading
 import time
+import traceback
 import unittest
 
-import yaml
-
 from packaging import version
 
 from concurrencytest import ConcurrentTestSuite, fork_for_tests
 from mininet.log import setLogLevel
 from mininet.clean import Cleanup
+import psutil
 
 from clib import mininet_test_util
+from clib.valve_test_lib import yaml_load, yaml_dump
 
 DEFAULT_HARDWARE = 'Open vSwitch'
 
@@ -58,8 +59,8 @@ SUPPORTS_METADATA = (
 
 
 EXTERNAL_DEPENDENCIES = (
-    ('ryu-manager', ['--version'],
-     'ryu-manager', r'ryu-manager (\d+\.\d+)\n', "4.9"),
+    ('osken-manager', ['--version'],
+     'osken-manager', r'osken-manager\s+(\d+\.\d+\.\d+)\n', "2.1"),
     ('ovs-vsctl', ['--version'], 'Open vSwitch',
      r'ovs-vsctl\s+\(Open vSwitch\)\s+(\d+\.\d+)\.\d+\n', "2.3"),
     ('tcpdump', ['-h'], 'tcpdump',
@@ -69,7 +70,7 @@ EXTERNAL_DEPENDENCIES = (
     ('fuser', ['-V'], r'fuser \(PSmisc\)',
      r'fuser \(PSmisc\) (\d+\.\d+|UNKNOWN)\n', "22.0"),
     ('lsof', ['-v'], r'lsof version',
-     r'revision: (\d+\.\d+)\n', "4.86"),
+     r'revision: (\d+\.\d+(\.\d+)?)\n', "4.86"),
     ('mn', ['--version'], r'\d+\.\d+.\d+',
      r'(\d+\.\d+).\d+', "2.2"),
     ('exabgp', ['--version'], 'ExaBGP',
@@ -128,8 +129,8 @@ def import_hw_config():
         print('Cannot find %s in %s' % (HW_SWITCH_CONFIG_FILE, CONFIG_FILE_DIRS))
         sys.exit(-1)
     try:
-        with open(config_file_name, 'r') as config_file:
-            config = yaml.safe_load(config_file)
+        with open(config_file_name, 'r', encoding='utf-8') as config_file:
+            config = yaml_load(config_file)
     except IOError:
         print('Could not load YAML config data from %s' % config_file_name)
         sys.exit(-1)
@@ -171,16 +172,16 @@ def check_dependencies():
         required_binary = 'required binary/library %s' % (
             ' '.join(binary_args))
         try:
-            proc = subprocess.Popen(
-                binary_args,
-                stdin=mininet_test_util.DEVNULL,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.STDOUT,
-                close_fds=True)
-            proc_out, proc_err = proc.communicate()
-            binary_output = proc_out.decode()
-            if proc_err is not None:
-                binary_output += proc_err.decode()
+            with subprocess.Popen(
+                    binary_args,
+                    stdin=mininet_test_util.DEVNULL,
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.STDOUT,
+                    close_fds=True) as proc:
+                proc_out, proc_err = proc.communicate()
+                binary_output = proc_out.decode()
+                if proc_err is not None:
+                    binary_output += proc_err.decode()
         except subprocess.CalledProcessError:
             # Might have run successfully, need to parse output
             pass
@@ -244,8 +245,8 @@ def pipeline_superset_report(decoded_pcap_logs):
         for flow_line, depth, section_stack in flow_lines:
             if depth == 1:
                 if flow_line.startswith('Type: OFPT_'):
-                    if not (flow_line.startswith('Type: OFPT_FLOW_MOD') or
-                            flow_line.startswith('Type: OFPT_GROUP_MOD')):
+                    if not (flow_line.startswith('Type: OFPT_FLOW_MOD')
+                            or flow_line.startswith('Type: OFPT_GROUP_MOD')):
                         return
                 if flow_line.startswith('Table ID'):
                     if not flow_line.startswith('Table ID: OFPTT_ALL'):
@@ -303,7 +304,8 @@ def pipeline_superset_report(decoded_pcap_logs):
     table_actions_max = collections.defaultdict(lambda: 0)
 
     for log in decoded_pcap_logs:
-        packets = re.compile(r'\n{2,}').split(open(log).read())
+        with open(log, encoding='utf-8') as log_file:
+            packets = re.compile(r'\n{2,}').split(log_file.read())
         for packet in packets:
             last_packet_line = None
             indent_count = 0
@@ -330,7 +332,6 @@ def pipeline_superset_report(decoded_pcap_logs):
                 flow_lines.append((packet_line, depth, copy.copy(section_stack)))
             parse_flow(flow_lines)
 
-
     for table in sorted(table_matches):
         print('table: %u' % table)
         print('  matches: %s (max %u)' % (
@@ -373,7 +374,7 @@ def filter_test_hardware(test_obj, hw_config):
 
 
 def max_loadavg():
-    return int(multiprocessing.cpu_count() * 1.5)
+    return int(psutil.cpu_count(logical=False) * 1.5)
 
 
 def expand_tests(modules, requested_test_classes, regex_test_classes, excluded_test_classes,
@@ -431,7 +432,7 @@ def expand_tests(modules, requested_test_classes, regex_test_classes, excluded_t
     return (sanity_tests, single_tests, parallel_tests)
 
 
-class FaucetResult(unittest.runner.TextTestResult): # pytype: disable=module-attr
+class FaucetResult(unittest.runner.TextTestResult):  # pytype: disable=module-attr
 
     root_tmpdir = None
     test_duration_secs = {}
@@ -446,7 +447,7 @@ class FaucetResult(unittest.runner.TextTestResult): # pytype: disable=module-att
         if test.id() not in self.test_duration_secs:
             self.test_duration_secs[test.id()] = 0
         try:
-            with open(duration_file_name) as duration_file:
+            with open(duration_file_name, encoding='utf-8') as duration_file:
                 self.test_duration_secs[test.id()] = int(duration_file.read())
         except FileNotFoundError:
             pass
@@ -472,8 +473,6 @@ class FaucetCleanupResult(FaucetResult):
 
 
 def debug_exception_handler(etype, value, trace):
-    import traceback
-    import pdb
     traceback.print_exception(etype, value, trace)
     print()
     pdb.pm()
@@ -529,7 +528,7 @@ def report_tests(test_status, test_list, result):
                 'status': test_status,
                 'output': test_text,
                 'test_duration_secs': test_duration_secs
-                }})
+            }})
     return tests_json
 
 
@@ -551,14 +550,13 @@ def report_results(results, hw_config, report_json_filename):
                     ('OK', result.successes))
             for test_status, test_list in test_lists:
                 tests_json.update(report_tests(test_status, test_list, result))
-        print(yaml.dump(
-            tests_json, default_flow_style=False, explicit_start=True, explicit_end=True))
+        print(yaml_dump(tests_json))
         if report_json_filename:
             report_json = {
                 'hw_config': hw_config,
                 'tests': tests_json,
             }
-            with open(report_json_filename, 'w') as report_json_file:
+            with open(report_json_filename, 'w', encoding='utf-8') as report_json_file:
                 report_json_file.write(json.dumps(report_json))
 
 
@@ -571,7 +569,8 @@ def run_test_suites(debug, report_json_filename, hw_config, root_tmpdir,
     results.extend(run_parallel_test_suites(root_tmpdir, resultclass, parallel_tests))
     results.extend(run_single_test_suites(debug, root_tmpdir, resultclass, single_tests))
     report_results(results, hw_config, report_json_filename)
-    successful_results = [result for result in results if result.wasSuccessful() or result.unexpected_success]
+    successful_results = [result for result in results
+                          if result.wasSuccessful() or result.unexpected_success]
     return len(results) == len(successful_results)
 
 
@@ -604,7 +603,8 @@ def dump_failed_test_file(test_file, only_exts):
 
     if dump_file:
         try:
-            test_file_content = open(test_file).read()
+            with open(test_file, encoding='utf-8') as test_file_h:
+                test_file_content = test_file_h.read()
             if test_file_content:
                 print(test_file)
                 print('=' * len(test_file))
@@ -655,6 +655,7 @@ def run_tests(modules, hw_config, requested_test_classes, regex_test_classes, du
         print('Testing hardware, forcing test serialization')
         serial = True
     root_tmpdir = tempfile.mkdtemp(prefix='faucet-tests-', dir='/var/tmp')
+    os.chmod(root_tmpdir, 0o755)
     print('Logging test results in %s' % root_tmpdir)
     start_free_ports = 10
     min_free_ports = 200
@@ -673,7 +674,9 @@ def run_tests(modules, hw_config, requested_test_classes, regex_test_classes, du
         modules, requested_test_classes, regex_test_classes, excluded_test_classes,
         hw_config, root_tmpdir, ports_sock, serial, port_order, start_port)
 
-    if sanity_tests.countTestCases() + single_tests.countTestCases() + parallel_tests.countTestCases():
+    test_count = (sanity_tests.countTestCases() + single_tests.countTestCases() + parallel_tests.countTestCases())
+
+    if test_count:
         no_tests = False
         sanity_result = run_sanity_test_suite(root_tmpdir, resultclass, sanity_tests)
         if sanity_result.wasSuccessful():
@@ -791,7 +794,7 @@ def parse_args():
         args.loglevel, args.profile)
 
 
-def test_main(modules):
+def test_main(modules, serial_override=None):
     """Test main."""
 
     print('testing module %s' % modules)
@@ -800,6 +803,10 @@ def test_main(modules):
      serial, repeat, excluded_test_classes, report_json_filename, port_order,
      start_port, loglevel, profile) = parse_args()
 
+    if serial_override is not None:
+        print('overriding serial to ', serial_override)
+        serial = serial_override
+
     setLogLevel(loglevel)
 
     if clean:
@@ -822,7 +829,8 @@ def test_main(modules):
     hw_config = import_hw_config()
 
     if profile:
-        pr = cProfile.Profile(time.time)  # use wall clock time
+        # use wall clock time
+        pr = cProfile.Profile(time.time)  # pylint: disable=invalid-name
         pr.enable()
 
     run_tests(
@@ -832,5 +840,5 @@ def test_main(modules):
 
     if profile:
         pr.disable()
-        ps = pstats.Stats(pr).sort_stats('cumulative')
+        ps = pstats.Stats(pr).sort_stats('cumulative')  # pylint: disable=invalid-name
         ps.print_stats()
