@@ -42,10 +42,10 @@ class FaucetLink(Link):
     """Link using FaucetIntfs"""
 
     def __init__(self, node1, node2, port1=None, port2=None,
-                 intfName1=None, intfName2=None,
+                 intf_name1=None, intf_name2=None,
                  addr1=None, addr2=None, **params):
         Link.__init__(self, node1, node2, port1=port1, port2=port2,
-                      intfName1=intfName1, intfName2=intfName2,
+                      intfName1=intf_name1, intfName2=intf_name2,
                       cls1=FaucetIntf, cls2=FaucetIntf,
                       addr1=addr1, addr2=addr2,
                       params1=params, params2=params)
@@ -54,13 +54,26 @@ class FaucetLink(Link):
 class FaucetHost(CPULimitedHost):
     """Base Mininet Host class, for Mininet-based tests."""
 
+    def __init__(self, *args, **kwargs):
+        self.pid_files = []
+        super().__init__(*args, **kwargs)
+
+    def terminate(self):
+        # If any 'dnsmasq' processes were started, terminate them now
+        for pid_file in self.pid_files:
+            with open(pid_file, 'r', encoding='utf-8') as pf:
+                for _, pid in enumerate(pf):
+                    os.kill(int(pid), 15)
+        super().terminate()
+
     def create_dnsmasq(self, tmpdir, iprange, router, vlan, interface=None):
         """Start dnsmasq instance inside dnsmasq namespace"""
         if interface is None:
             interface = self.defaultIntf()
-        dhcp_leasefile = os.path.join(tmpdir, 'nfv-dhcp-%s-vlan%u.leases' % (self.name, vlan))
-        log_facility = os.path.join(tmpdir, 'nfv-dhcp-%s-vlan%u.log' % (self.name, vlan))
-        pid_file = os.path.join(tmpdir, 'dnsmasq-%s-vlan%u.pid' % (self.name, vlan))
+        dhcp_leasefile = os.path.join(tmpdir, 'nfv-dhcp-%s-%s-vlan%u.leases' % (self.name, iprange, vlan))
+        log_facility = os.path.join(tmpdir, 'nfv-dhcp-%s-%s-vlan%u.log' % (self.name, iprange, vlan))
+        pid_file = os.path.join(tmpdir, 'dnsmasq-%s-%s-vlan%u.pid' % (self.name, iprange, vlan))
+        self.pid_files.append(pid_file)
         cmd = 'dnsmasq'
         opts = ''
         opts += ' --dhcp-range=%s,255.255.255.0' % iprange
@@ -69,13 +82,27 @@ class FaucetHost(CPULimitedHost):
         opts += ' --no-resolv --txt-record=does.it.work,yes'
         opts += ' --bind-interfaces'
         opts += ' --except-interface=lo'
-        opts += ' --interface=%s' % (interface)
+        opts += ' --interface=%s' % interface
         opts += ' --dhcp-leasefile=%s' % dhcp_leasefile
         opts += ' --log-facility=%s' % log_facility
         opts += ' --pid-file=%s' % pid_file
         opts += ' --conf-file='
         return self.cmd(cmd + opts)
 
+    def run_dhclient(self, tmpdir, interface=None, timeout=10):
+        """Run DHCLIENT to obtain ip address via DHCP"""
+        if interface is None:
+            interface = self.defaultIntf()
+        cmd = 'dhclient'
+        opts = ''
+        opts += ' -1'
+        opts += ' -d'
+        opts += ' -pf %s/dhclient-%s.pid' % (tmpdir, self.name)
+        opts += ' -lf %s/dhclient-%s.leases' % (tmpdir, self.name)
+        opts += ' %s' % interface
+        dhclient_cmd = cmd + opts
+        return self.cmd(mininet_test_util.timeout_cmd(dhclient_cmd, timeout), verbose=True)
+
     def return_ip(self):
         """Return host IP as a string"""
         return self.cmd('hostname -I')
@@ -89,11 +116,16 @@ class VLANHost(FaucetHost):
     vlans = None
     vlan_intfs = None
 
-    def config(self, vlans=[100], **params):  # pylint: disable=arguments-differ
+    def config(self, vlans=None, **params):  # pylint: disable=arguments-differ
         """Configure VLANHost according to (optional) parameters:
-           vlans (list): List of VLAN IDs (for the VLANs the host is configured to have) for default interface
-           vlan_intfs (dict): Dictionary of interface IP addresses keyed by VLAN indices"""
+
+        vlans (list): List of VLAN IDs (for the VLANs the host is configured to have)
+            for default interface
+        vlan_intfs (dict): Dictionary of interface IP addresses keyed by VLAN indices
+        """
         super_config = super().config(**params)
+        if vlans is None:
+            vlans = [100]
         self.vlans = vlans
         self.vlan_intfs = {}
         cmds = []
@@ -101,7 +133,7 @@ class VLANHost(FaucetHost):
         self.intf_root_name = intf.name
         if 'vlan_intfs' in params:
             vlan_intfs = params.get('vlan_intfs', {})
-            for vlan_id, ip in vlan_intfs.items():
+            for vlan_id, ip_addr in vlan_intfs.items():
                 if isinstance(vlan_id, tuple):
                     # Interface will take multiply VLAN tagged packets
                     intf_name = '%s' % intf.name
@@ -110,18 +142,19 @@ class VLANHost(FaucetHost):
                         # Cannot have intf name tu0xy-eth0.VID1.VID2 as that takes up too many bytes
                         intf_name += '.%s' % vlan_i
                         cmds.extend([
-                            'ip link add link %s name %s type vlan id %s' % (prev_name, intf_name, vlans[vlan_i]),
+                            'ip link add link %s name %s type vlan id %s' %
+                            (prev_name, intf_name, vlans[vlan_i]),
                             'ip link set dev %s up' % (intf_name)
                         ])
                         self.nameToIntf[intf_name] = intf
                         self.vlan_intfs.setdefault(vlan_id, [])
                         self.vlan_intfs[vlan_id].append(intf_name)
-                    cmds.append('ip -4 addr add %s dev %s' % (ip, intf_name))
+                    cmds.append('ip -4 addr add %s dev %s' % (ip_addr, intf_name))
                 else:
                     intf_name = '%s.%s' % (intf, vlans[vlan_id])
                     cmds.extend([
                         'vconfig add %s %d' % (intf.name, vlans[vlan_id]),
-                        'ip -4 addr add %s dev %s' % (ip, intf_name),
+                        'ip -4 addr add %s dev %s' % (ip_addr, intf_name),
                         'ip link set dev %s up' % intf_name])
                     self.nameToIntf[intf_name] = intf
                     self.vlan_intfs[vlan_id] = intf_name
@@ -130,8 +163,8 @@ class VLANHost(FaucetHost):
             cmds.extend([
                 'ip link set dev %s up' % vlan_intf_name,
                 'ip -4 addr add %s dev %s' % (params['ip'], vlan_intf_name)])
-            for v in vlans:
-                cmds.append('vconfig add %s %d' % (intf, v))
+            for vlan in vlans:
+                cmds.append('vconfig add %s %d' % (intf, vlan))
             intf.name = vlan_intf_name
             self.nameToIntf[vlan_intf_name] = intf
         cmds.extend([
@@ -164,8 +197,8 @@ class FaucetSwitch(OVSSwitch):
         # Workaround: ignore ethtool errors on tap interfaces
         # This allows us to use tap tunnels as cables to switch ports,
         # for example to test against OvS in a VM.
-        if (len(args) > 1 and args[0] == 'ethtool -K' and
-                getattr(args[1], 'name', '').startswith('tap')):
+        if (len(args) > 1 and args[0] == 'ethtool -K'
+                and getattr(args[1], 'name', '').startswith('tap')):
             return True
         return False
 
@@ -191,7 +224,7 @@ class FaucetSwitch(OVSSwitch):
         port = self.ports[intf]
         self.cmd('ovs-vsctl set Interface', intf, 'ofport_request=%s' % port)
 
-    def addController(self, controller):
+    def add_controller(self, controller):
         self.clist.append((
             self.name + controller.name, '%s:%s:%d' % (
                 controller.protocol, controller.IP(), controller.port)))
@@ -208,8 +241,8 @@ class FaucetSwitch(OVSSwitch):
         # Controller ID list
         cids = ','.join('@%s' % name for name, _target in self.clist)
         # One ovs-vsctl command to rule them all!
-        self.vsctl(cargs +
-                   ' -- set bridge %s controller=[%s]' % (self, cids))
+        self.vsctl(cargs
+                   + ' -- set bridge %s controller=[%s]' % (self, cids))
 
     def start(self, controllers):
         # Transcluded from Mininet source, since need to insert
@@ -217,16 +250,16 @@ class FaucetSwitch(OVSSwitch):
         int(self.dpid, 16)  # DPID must be a hex string
         switch_intfs = [intf for intf in self.intfList() if self.ports[intf] and not intf.IP()]
         # Command to add interfaces
-        intfs = ' '.join(' -- add-port %s %s' % (self, intf) +
-                         self.intfOpts(intf)
+        intfs = ' '.join(' -- add-port %s %s' % (self, intf)
+                         + self.intfOpts(intf)
                          for intf in switch_intfs)
         # Command to create controller entries
         self.clist = [(self.name + c.name, '%s:%s:%d' %
-                  (c.protocol, c.IP(), c.port))
-                 for c in controllers]
+                       (c.protocol, c.IP(), c.port))
+                      for c in controllers]
         if self.listenPort:
             self.clist.append((self.name + '-listen',
-                          'ptcp:%s' % self.listenPort))
+                               'ptcp:%s' % self.listenPort))
         ccmd = '-- --id=@%s create Controller target=\\"%s\\"'
         if self.reconnectms:
             ccmd += ' max_backoff=%d' % self.reconnectms
@@ -240,11 +273,11 @@ class FaucetSwitch(OVSSwitch):
         if not self.isOldOVS():
             cargs += ' -- --if-exists del-br %s' % self
         # One ovs-vsctl command to rule them all!
-        self.vsctl(cargs +
-                   ' -- add-br %s' % self +
-                   ' -- set bridge %s controller=[%s]' % (self, cids) +
-                   self.bridgeOpts() +
-                   intfs)
+        self.vsctl(cargs
+                   + ' -- add-br %s' % self
+                   + ' -- set bridge %s controller=[%s]' % (self, cids)
+                   + self.bridgeOpts()
+                   + intfs)
         # switch interfaces on mininet host, must have no IP config.
         for intf in switch_intfs:
             for ipv in (4, 6):
@@ -292,10 +325,10 @@ class FaucetSwitchTopo(Topo):
         return self.addHost(
             name=host_name, cls=VLANHost, vlans=tagged_vids, cpu=self.CPUF)
 
-    def _add_untagged_host(self, sid_prefix, host_n, inNamespace=True): # pylint: disable=invalid-name
+    def _add_untagged_host(self, sid_prefix, host_n, in_namespace=True):
         """Add a single untagged test host."""
         host_name = 'u%s%1.1u' % (sid_prefix, host_n + 1)
-        return self.addHost(name=host_name, cls=FaucetHost, cpu=self.CPUF, inNamespace=inNamespace)
+        return self.addHost(name=host_name, cls=FaucetHost, cpu=self.CPUF, inNamespace=in_namespace)
 
     def _add_extended_host(self, sid_prefix, host_n, e_cls, tmpdir):
         """Add a single extended test host."""
@@ -379,11 +412,11 @@ class FaucetSwitchTopo(Topo):
               get_serialno=mininet_test_util.get_serialno):
         if not host_namespace:
             host_namespace = {}
-        self.hw_dpid = hw_dpid  # pylint: disable=attribute-defined-outside-init
-        self.hw_ports = sorted(switch_map) if switch_map else []  # pylint: disable=attribute-defined-outside-init
-        self.start_port = start_port  # pylint: disable=attribute-defined-outside-init
+        self.hw_dpid = hw_dpid
+        self.hw_ports = sorted(switch_map) if switch_map else []
+        self.start_port = start_port
         maxlength = n_tagged + n_untagged + n_extended
-        self.port_order = self.extend_port_order(  # pylint: disable=attribute-defined-outside-init
+        self.port_order = self.extend_port_order(
             port_order, maxlength)
         for dpid in dpids:
             serialno = get_serialno(ports_sock, test_name)
@@ -392,7 +425,7 @@ class FaucetSwitchTopo(Topo):
                       for host_n in range(n_tagged)]
             untagged = [self._add_untagged_host(
                 sid_prefix, host_n, host_namespace.get(host_n, True))
-                        for host_n in range(n_untagged)]
+                for host_n in range(n_untagged)]
             extended = [self._add_extended_host(sid_prefix, host_n, e_cls, tmpdir)
                         for host_n in range(n_extended)]
             switch = self._add_faucet_switch(sid_prefix, dpid, hw_dpid, ovs_type)
@@ -427,12 +460,13 @@ socket_timeout=15
 
     def __init__(self, name, tmpdir, controller_intf=None, controller_ipv6=False,
                  cargs='', **kwargs):
-        name = '%s-%u' % (name, os.getpid())
+        self.name_no_pid = name
+        name_with_pid = '%s-%u' % (name, os.getpid())
         self.tmpdir = tmpdir
         self.controller_intf = controller_intf
         self.controller_ipv6 = controller_ipv6
         super().__init__(
-            name, cargs=self._add_cargs(cargs, name), **kwargs)
+            name_with_pid, cargs=self._add_cargs(cargs, name_with_pid), **kwargs)
 
     def _add_cargs(self, cargs, name):
         ofp_listen_host_arg = ''
@@ -440,13 +474,13 @@ socket_timeout=15
             socket_type = socket.AF_INET
             if self.controller_ipv6:
                 socket_type = socket.AF_INET6
-            self.controller_ip = netifaces.ifaddresses( # pylint: disable=c-extension-no-member
+            self.controller_ip = netifaces.ifaddresses(  # pylint: disable=c-extension-no-member
                 self.controller_intf)[socket_type][0]['addr']
             ofp_listen_host_arg = '--ryu-ofp-listen-host=%s' % self.controller_ip
         self.pid_file = os.path.join(self.tmpdir, name + '.pid')
         pid_file_arg = '--ryu-pid-file=%s' % self.pid_file
         ryu_conf_file = os.path.join(self.tmpdir, 'ryu.conf')
-        with open(ryu_conf_file, 'w') as ryu_conf:
+        with open(ryu_conf_file, 'w', encoding='utf-8') as ryu_conf:
             ryu_conf.write(self.RYU_CONF)
         ryu_conf_arg = '--ryu-config-file=%s' % ryu_conf_file
         return ' '.join((
@@ -466,7 +500,7 @@ socket_timeout=15
             '-n',
             '-U',
             '-q',
-            '-W 1', # max files 1
+            '-W 1',  # max files 1
             '-G %u' % (self.MAX_CTL_TIME - 1),
             '-c %u' % (self.MAX_OF_PKTS),
             '-i %s' % self.controller_intf,
@@ -506,7 +540,7 @@ socket_timeout=15
         if self.CPROFILE:
             cprofile_args = 'python3 -m cProfile -s time'
         full_faucet_dir = os.path.abspath(mininet_test_util.FAUCET_DIR)
-        with open(script_wrapper_name, 'w') as script_wrapper:
+        with open(script_wrapper_name, 'w', encoding='utf-8') as script_wrapper:
             faucet_cli = (
                 'PYTHONPATH=%s %s exec timeout %u %s %s %s $*\n' % (
                     os.path.dirname(full_faucet_dir),
@@ -522,7 +556,7 @@ socket_timeout=15
         """Return PID of ryu-manager process."""
         if os.path.exists(self.pid_file) and os.path.getsize(self.pid_file) > 0:
             pid = None
-            with open(self.pid_file) as pid_file:
+            with open(self.pid_file, encoding='utf-8') as pid_file:
                 pid = int(pid_file.read())
             return pid
         return None
@@ -558,9 +592,9 @@ socket_timeout=15
 
     def healthy(self):
         """Return True if controller logging and listening on required ports."""
-        if (os.path.exists(self.logname()) and
-                os.path.getsize(self.logname()) and
-                self.listening()):
+        if (os.path.exists(self.logname())
+                and os.path.getsize(self.logname())
+                and self.listening()):
             return True
         return False
 
@@ -572,9 +606,9 @@ socket_timeout=15
     def _stop_cap(self):
         """Stop tcpdump for OF port and run tshark to decode it."""
         if os.path.exists(self.ofcap):
-            self.cmd(' '.join(['fuser', '-15', '-m', self.ofcap]))
+            self.cmd(' '.join(['fuser', '-15', '-k', self.ofcap]))
             text_ofcap_log = '%s.txt' % self.ofcap
-            with open(text_ofcap_log, 'w') as text_ofcap:
+            with open(text_ofcap_log, 'w', encoding='utf-8') as text_ofcap:
                 subprocess.call(
                     ['timeout', str(self.MAX_CTL_TIME),
                      'tshark', '-l', '-n', '-Q',
@@ -600,7 +634,7 @@ socket_timeout=15
         super().stop()
         if os.path.exists(self.logname()):
             tmpdir_logname = os.path.join(
-                self.tmpdir, os.path.basename(self.logname()))
+                self.tmpdir, self.name + '-stdout-stderr.log')
             if os.path.exists(tmpdir_logname):
                 os.remove(tmpdir_logname)
             shutil.move(self.logname(), tmpdir_logname)
@@ -609,19 +643,18 @@ socket_timeout=15
 class FAUCET(BaseFAUCET):
     """Start a FAUCET controller."""
 
-    START_ARGS = ['--ryu-app=ryu.app.ofctl_rest']
+    START_ARGS = ['--ryu-app-lists=%s' % (os.path.dirname(os.path.realpath(__file__)) + '/../ofctl_rest/ofctl_rest.py')]
 
-    # pylint: disable=too-many-locals
     def __init__(self, name, tmpdir, controller_intf, controller_ipv6, env,
                  ctl_privkey, ctl_cert, ca_certs,
                  ports_sock, prom_port, port, test_name, **kwargs):
         self.prom_port = prom_port
         self.ofctl_port = mininet_test_util.find_free_port(
             ports_sock, test_name)
-        cargs = ' '.join((
-            '--ryu-wsapi-host=%s' % mininet_test_util.LOCALHOSTV6,
-            '--ryu-wsapi-port=%u' % self.ofctl_port,
-            self._tls_cargs(port, ctl_privkey, ctl_cert, ca_certs)))
+        env['OFCTL_PORT'] = str(self.ofctl_port)
+        env['OFCTL_HOST'] = mininet_test_util.LOCALHOSTV6
+        cargs = ' '.join(
+            self._tls_cargs(port, ctl_privkey, ctl_cert, ca_certs))
         super().__init__(
             name,
             tmpdir,
@@ -634,9 +667,9 @@ class FAUCET(BaseFAUCET):
 
     def listening(self):
         return (
-            self.listen_port(self.ofctl_port) and
-            self.listen_port(self.prom_port) and
-            super().listening())
+            self.listen_port(self.ofctl_port)
+            and self.listen_port(self.prom_port)
+            and super().listening())
 
 
 class Gauge(BaseFAUCET):
