@@ -31,6 +31,7 @@ class TcpdumpHelper:
             self.intf_name = self.intf_name.split('.')[0]
 
         tcpdump_flags = vflags
+        tcpdump_flags += ' -Z root'
         tcpdump_flags += ' -c %u' % packets if packets else ''
         tcpdump_flags += ' -w %s' % pcap_out if pcap_out else ''
         tcpdump_cmd = 'tcpdump -i %s %s --immediate-mode -e -n -U %s' % (
@@ -48,9 +49,10 @@ class TcpdumpHelper:
             close_fds=True,
             shell=False)
 
-        if self.stream():
+        stream = self.stream()
+        if stream:
             debug('tcpdump_helper stream fd %s %s' % (
-                self.stream().fileno(), self.intf_name))
+                stream.fileno(), self.intf_name))
 
         self.readbuf = ''
         self.set_blocking(blocking)
@@ -75,22 +77,24 @@ class TcpdumpHelper:
     def execute(self):
         """Run the helper and accumulate tcpdump output."""
         tcpdump_txt = ''
-        if self.stream():
+        stream = self.stream()
+        if stream:
             while True:
                 line = self.next_line()
                 if not line:
                     break
-                debug('tcpdump_helper fd %d line "%s"' % (self.stream().fileno(), line))
+                debug('tcpdump_helper fd %d line "%s"' % (stream.fileno(), line))
                 tcpdump_txt += line.strip()
         return tcpdump_txt
 
     def terminate(self):
         """Terminate the helper."""
-        if not self.pipe or not self.stream():
+        stream = self.stream()
+        if not self.pipe or not stream:
             return -1
 
         try:
-            debug('tcpdump_helper terminate fd %s' % self.stream().fileno())
+            debug('tcpdump_helper terminate fd %s' % stream.fileno())
             self.pipe.terminate()
             result = self.pipe.wait()
             if result == 124:
