@@ -2,15 +2,16 @@
 
 set -euo pipefail
 
-APK="apk -q"
-BUILDDEPS="gcc python3-dev musl-dev parallel yaml-dev g++"
-TESTDEPS="bitstring pytest wheel virtualenv"
-PIP3="pip3 -q --no-cache-dir install --upgrade"
+APK="apk --no-cache"
+BUILDDEPS="git gcc python3-dev musl-dev parallel yaml-dev g++"
+TESTDEPS="bitstring pytest wheel virtualenv pip"
+PIP3="pip3 install --upgrade"
 FROOT="/faucet-src"
 
 dir=$(dirname "$0")
 
-${APK} add -U git ${BUILDDEPS}
+${APK} add -U ${BUILDDEPS}
+"${dir}/retrycmd.sh" "${PIP3} git+https://github.com/faucetsdn/python3-fakencclient"
 "${dir}/retrycmd.sh" "${PIP3} ${TESTDEPS}"
 "${dir}/retrycmd.sh" "${PIP3} -r ${FROOT}/requirements.txt"
 ${PIP3} ${FROOT}
@@ -32,7 +33,11 @@ for i in ${BUILDDEPS} ; do
 done
 
 # Clean up
+rm -r "${HOME}/.cache"
 rm -r "${FROOT}"
+rm -r /usr/local/lib/python3*/site-packages/os_ken/tests/
+rm -r /usr/local/lib/python3*/site-packages/os_ken/lib/of_config/
+rm /usr/local/lib/python3*/site-packages/os_ken/cmd/of_config_cli.py
 
 # Smoke test
 faucet -V || exit 1
