@@ -75,7 +75,8 @@ def build_ordered_output_actions(acl_table, output_list, tunnel_rules=None, sour
                 output_actions.append(valve_of.group_act(group_id=group_id))
             if key == 'tunnel' and tunnel_rules and source_id is not None:
                 source_rule = tunnel_rules[value][source_id]
-                _, tunnel_actions, tunnel_ofmsgs, tunnel_inst = build_output_actions(acl_table, source_rule)
+                _, tunnel_actions, tunnel_ofmsgs, tunnel_inst = build_output_actions(acl_table,
+                                                                                     source_rule)
                 output_actions.extend(tunnel_actions)
                 output_ofmsgs.extend(tunnel_ofmsgs)
                 output_inst.extend(tunnel_inst)
@@ -104,6 +105,38 @@ def rewrite_vlan(acl_table, output_dict):
     return vlan_actions
 
 
+def build_ct_actions(acl_table, ct_dict):
+    """Build conntrack action from ACL rule"""
+
+    if 'clear' in ct_dict and ct_dict['clear']:
+        return valve_of.ct_clear()
+
+    ct_actions = []
+
+    if 'nat' in ct_dict:
+        nat_args = {
+            'flags': ct_dict['nat'].get('flags', 0),
+            'range_ipv4_min': ct_dict['nat'].get('range_ipv4_min', ''),
+            'range_ipv4_max': ct_dict['nat'].get('range_ipv4_max', ''),
+            'range_ipv6_min': ct_dict['nat'].get('range_ipv6_min', ''),
+            'range_ipv6_max': ct_dict['nat'].get('range_ipv6_max', ''),
+            'range_proto_min': ct_dict['nat'].get('range_proto_min', None),
+            'range_proto_max': ct_dict['nat'].get('range_proto_max', None),
+        }
+        ct_actions.append(valve_of.ct_nat(**nat_args))
+
+    ct_args = {
+        'flags': ct_dict.get('flags', 0),
+        'actions': ct_actions,
+        'alg': ct_dict.get('alg', 0),
+        'recirc_table': ct_dict.get('table'),
+        'zone_ofs_nbits': ct_dict.get('zone'),
+        'zone_src': ct_dict.get('zone_src', None),
+    }
+
+    return valve_of.ct(**ct_args)
+
+
 def build_output_actions(acl_table, output_dict, tunnel_rules=None, source_id=None):
     """Implement actions to alter packet/output."""
     if isinstance(output_dict, (list, tuple)):
@@ -194,6 +227,9 @@ def build_acl_entry(  # pylint: disable=too-many-arguments,too-many-branches,too
                 # if port specified, output packet now and exit pipeline.
                 if not allow and output_port is not None:
                     continue
+            if 'ct' in attrib_value:
+                ct_action = build_ct_actions(acl_table, attrib_value['ct'])
+                acl_act.append(ct_action)
 
             if allow:
                 acl_inst.extend(allow_inst)
@@ -207,8 +243,8 @@ def build_acl_entry(  # pylint: disable=too-many-arguments,too-many-branches,too
         acl_match_dict['vlan_vid'] = valve_of.vid_present(vlan_vid)
     try:
         acl_match = valve_of.match_from_dict(acl_match_dict)
-    except TypeError:
-        raise InvalidConfigError('invalid match type in ACL')
+    except TypeError as type_error:
+        raise InvalidConfigError('invalid match type in ACL') from type_error
     if acl_act:
         acl_inst.append(valve_of.apply_actions(acl_act))
     return (acl_match, acl_inst, acl_cookie, acl_ofmsgs)
@@ -236,8 +272,8 @@ def build_tunnel_ofmsgs(rule_conf, acl_table, priority,
         acl_match_dict['vlan_pcp'] = valve_of.PCP_TUNNEL_FLAG
     try:
         acl_match = valve_of.match_from_dict(acl_match_dict)
-    except TypeError:
-        raise InvalidConfigError('invalid match type in ACL')
+    except TypeError as type_error:
+        raise InvalidConfigError('invalid match type in ACL') from type_error
     flowmod = acl_table.flowmod(acl_match, priority=priority, inst=tuple(acl_inst))
     if flowdel:
         ofmsgs.append(acl_table.flowdel(match=acl_match, priority=priority, strict=False))
@@ -379,8 +415,16 @@ class ValveAclManager(ValveManagerBase):
         ofmsgs.extend(self.add_port(port))
         return ofmsgs
 
+    def del_vlan(self, vlan):
+        """Remove VLAN ACLs if configured."""
+        ofmsgs = []
+        for table in (self.vlan_acl_table, self.egress_acl_table):
+            if table is not None:
+                ofmsgs.append(table.flowdel(match=table.match(vlan=vlan)))
+        return ofmsgs
+
     def add_vlan(self, vlan, cold_start):
-        """Install vlan ACLS if configured"""
+        """Install VLAN ACL if configured."""
         ofmsgs = []
         if vlan.acls_in:
             acl_allow_inst = self.pipeline.accept_to_classification()
@@ -396,13 +440,13 @@ class ValveAclManager(ValveManagerBase):
                     vlan.acls_out, self.egress_acl_table, egress_acl_allow_inst,
                     egress_acl_allow_inst, self.acl_priority, self.meters,
                     vlan.acls_out[0].exact_match, vlan_vid=vlan.vid
-                    ))
+                ))
             else:
                 ofmsgs.append(self.egress_acl_table.flowmod(
                     self.egress_acl_table.match(vlan=vlan),
                     priority=self.acl_priority,
                     inst=tuple(egress_acl_allow_inst),
-                    ))
+                ))
         return ofmsgs
 
     def add_authed_mac(self, port_num, mac):
@@ -490,14 +534,14 @@ class ValveAclManager(ValveManagerBase):
                     eth_type=valve_packet.ETH_EAPOL,
                     eth_src=mac),
                 priority=self.override_priority,
-                ),
+            ),
             self.port_acl_table.flowdel(
                 match=self.port_acl_table.match(
                     in_port=port_num,
                     eth_type=valve_packet.ETH_EAPOL),
                 priority=self.override_priority,
-                )
-            ]
+            )
+        ]
         return ofmsgs
 
     def create_mab_flow(self, port_num, nfv_sw_port_num, mac):
