@@ -1,5 +1,7 @@
 """Utility functions to parse/create OpenFlow messages."""
 
+# pylint: disable=too-many-lines
+
 # Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
 # Copyright (C) 2015 Brad Cowie, Christopher Lorier and Joe Stringer.
 # Copyright (C) 2015 Research and Education Advanced Network New Zealand Ltd.
@@ -21,14 +23,16 @@ import functools
 import ipaddress
 import random
 
-from ryu.lib import mac
-from ryu.lib import ofctl_v1_3 as ofctl
-from ryu.lib.ofctl_utils import (
+# pylint: disable=unused-import
+from os_ken.lib import mac  # noqa: F401
+from os_ken.lib import ofctl_v1_3 as ofctl
+from os_ken.lib.ofctl_utils import (
     str_to_int, to_match_ip, to_match_masked_int, to_match_eth, to_match_vid, OFCtlUtil)
-from ryu.ofproto import ether
-from ryu.ofproto import inet
-from ryu.ofproto import ofproto_v1_3 as ofp
-from ryu.ofproto import ofproto_v1_3_parser as parser
+from os_ken.ofproto import ether
+# pylint: disable=unused-import
+from os_ken.ofproto import inet  # noqa: F401
+from os_ken.ofproto import ofproto_v1_3 as ofp
+from os_ken.ofproto import ofproto_v1_3_parser as parser
 
 from faucet.conf import test_config_condition, InvalidConfigError
 from faucet.valve_of_old import OLD_MATCH_FIELDS
@@ -46,9 +50,11 @@ ECTP_ETH_TYPE = 0x9000
 # Avoid use of PCP 1 which is BK priority (lowest)
 
 TUNNEL_INDICATOR_FIELD = 'vlan_pcp'
-# Used to indicate traffic in a one-to-one bi-directional tunnel is heading in the reverse/return direction
+# Used to indicate traffic in a one-to-one bi-directional tunnel is heading
+# in the reverse/return direction
 PCP_TUNNEL_REVERSE_DIRECTION_FLAG = 4
-# Used to indicate traffic belongs in a tunnel (for all cases not including the 1-1 reverse bi-directional tunnel)
+# Used to indicate traffic belongs in a tunnel (for all cases not including
+# the 1-1 reverse bi-directional tunnel)
 PCP_TUNNEL_FLAG = 3
 
 PCP_EXT_PORT_FLAG = 2
@@ -303,8 +309,8 @@ def is_groupdel(ofmsg):
     Returns:
         bool: True if is a GroupMod delete
     """
-    if (is_groupmod(ofmsg) and
-            (ofmsg.command == ofp.OFPGC_DELETE)):
+    if (is_groupmod(ofmsg)
+            and (ofmsg.command == ofp.OFPGC_DELETE)):
         return True
     return False
 
@@ -317,8 +323,8 @@ def is_meterdel(ofmsg):
     Returns:
         bool: True if is a MeterMod delete
     """
-    if (is_metermod(ofmsg) and
-            (ofmsg.command == ofp.OFPMC_DELETE)):
+    if (is_metermod(ofmsg)
+            and (ofmsg.command == ofp.OFPMC_DELETE)):
         return True
     return False
 
@@ -331,8 +337,8 @@ def is_groupadd(ofmsg):
     Returns:
         bool: True if is a GroupMod add
     """
-    if (is_groupmod(ofmsg) and
-            (ofmsg.command == ofp.OFPGC_ADD)):
+    if (is_groupmod(ofmsg)
+            and (ofmsg.command == ofp.OFPGC_ADD)):
         return True
     return False
 
@@ -345,8 +351,8 @@ def is_meteradd(ofmsg):
     Returns:
         bool: True if is a MeterMod add
     """
-    if (is_metermod(ofmsg) and
-            (ofmsg.command == ofp.OFPMC_ADD)):
+    if (is_metermod(ofmsg)
+            and (ofmsg.command == ofp.OFPMC_ADD)):
         return True
     return False
 
@@ -359,8 +365,8 @@ def is_apply_actions(instruction):
     Returns:
         bool: True if an apply action.
     """
-    return (isinstance(instruction, parser.OFPInstructionActions) and
-            instruction.type == ofp.OFPIT_APPLY_ACTIONS)
+    return (isinstance(instruction, parser.OFPInstructionActions)
+            and instruction.type == ofp.OFPIT_APPLY_ACTIONS)
 
 
 def is_meter(instruction):
@@ -378,6 +384,10 @@ def is_set_field(action):
     return isinstance(action, parser.OFPActionSetField)
 
 
+def is_ct(action):
+    return isinstance(action, parser.NXActionCT)  # pylint: disable=no-member
+
+
 def apply_meter(meter_id):
     """Return instruction to apply a meter."""
     return parser.OFPInstructionMeter(meter_id, ofp.OFPIT_METER)
@@ -435,7 +445,7 @@ def metadata_goto_table(metadata, mask, table):
     return [
         parser.OFPInstructionWriteMetadata(metadata, mask),
         parser.OFPInstructionGotoTable(table.table_id)
-        ]
+    ]
 
 
 @functools.lru_cache()
@@ -507,6 +517,39 @@ def pop_vlan():
     return parser.OFPActionPopVlan()
 
 
+def ct(**kwds):  # pylint: disable=invalid-name
+    """Return connection tracker action.
+
+    Args:
+        kwds (dict): exactly one connection tracker action.
+    Returns:
+        ryu.ofproto.nx_actions.NXActionCT: connection tracker action.
+    """
+    return parser.NXActionCT(**kwds)  # pylint: disable=no-member
+
+
+def ct_clear():
+    """Return clear connection tracker state action.
+
+    Args:
+        kwds (dict): exactly one clear connection tracker state action.
+    Returns:
+        ryu.ofproto.nx_actions.NXActionCTClear: clear connection tracker state action.
+    """
+    return parser.NXActionCTClear()  # pylint: disable=no-member
+
+
+def ct_nat(**kwds):
+    """Return network address translation connection tracker action.
+
+    Args:
+        kwds (dict): exactly one network address translation connection tracker action.
+    Returns:
+        ryu.ofproto.nx_actions.NXActionNAT: network address translation connection tracker action.
+    """
+    return parser.NXActionNAT(**kwds)  # pylint: disable=no-member
+
+
 @functools.lru_cache(maxsize=1024)
 def output_port(port_num, max_len=0):
     """Return OpenFlow action to output to a port.
@@ -656,8 +699,8 @@ def valve_match_vid(value):
     return to_match_vid(value, ofp.OFPVID_PRESENT)
 
 
-# See 7.2.3.7 Flow Match Fields (OF 1.3.5)
 MATCH_FIELDS = {
+    # See 7.2.3.7 Flow Match Fields (OF 1.3.5)
     'in_port': OFCtlUtil(ofp).ofp_port_from_user,
     'in_phy_port': str_to_int,
     'metadata': to_match_masked_int,
@@ -697,7 +740,13 @@ MATCH_FIELDS = {
     'mpls_bos': str_to_int,
     'pbb_isid': to_match_masked_int,
     'tunnel_id': to_match_masked_int,
-    'ipv6_exthdr': to_match_masked_int
+    'ipv6_exthdr': to_match_masked_int,
+
+    # Nicira extensions, see ovs-fields(7)
+    'ct_state': to_match_masked_int,
+    'ct_zone': str_to_int,
+    'ct_mark': to_match_masked_int,
+    'ct_label': to_match_masked_int
 }
 
 
@@ -709,8 +758,9 @@ def match_from_dict(match_dict):
         test_config_condition(of_match not in MATCH_FIELDS, 'Unknown match field: %s' % of_match)
         try:
             encoded_field = MATCH_FIELDS[of_match](field)
-        except TypeError:
-            raise InvalidConfigError('%s cannot be type %s' % (of_match, type(field)))
+        except TypeError as type_error:
+            raise InvalidConfigError('%s cannot be type %s' %
+                                     (of_match, type(field))) from type_error
         kwargs[of_match] = encoded_field
 
     return parser.OFPMatch(**kwargs)
@@ -829,7 +879,7 @@ def build_group_flood_buckets(vlan_flood_acts):
     non_outputs = []
     for act in vlan_flood_acts:
         if is_output(act):
-            buckets.append(bucket(actions=non_outputs+[act]))
+            buckets.append(bucket(actions=non_outputs + [act]))
         else:
             non_outputs.append(act)
     return buckets
@@ -1005,7 +1055,7 @@ def sort_flows(input_ofmsgs):
         input_ofmsgs,
         key=lambda ofmsg: (
             getattr(ofmsg, 'table_id', ofp.OFPTT_ALL),
-            getattr(ofmsg, 'priority', 2**16+1)), reverse=True)
+            getattr(ofmsg, 'priority', 2**16 + 1)), reverse=True)
 
 
 def dedupe_ofmsgs(input_ofmsgs, random_order, flowkey):
@@ -1148,8 +1198,8 @@ def flood_untagged_port_outputs(ports, in_port=None, exclude_ports=None):
 def flood_port_outputs(tagged_ports, untagged_ports, in_port=None, exclude_ports=None):
     """Return actions for both tagged and untagged ports."""
     return (
-        flood_tagged_port_outputs(tagged_ports, in_port, exclude_ports) +
-        flood_untagged_port_outputs(untagged_ports, in_port, exclude_ports))
+        flood_tagged_port_outputs(tagged_ports, in_port, exclude_ports)
+        + flood_untagged_port_outputs(untagged_ports, in_port, exclude_ports))
 
 
 def faucet_config(datapath=None):
