@@ -48,14 +48,14 @@ class ValvePipeline(ValveManagerBase):
         self.select_priority = self._HIGH_PRIORITY
 
     @staticmethod
-    @functools.lru_cache()
+    @functools.lru_cache(maxsize=1024)
     def _accept_to_table(table, actions):
         inst = [table.goto_this()]
         if actions:
             inst.append(valve_of.apply_actions(actions))
         return tuple(inst)
 
-    @functools.lru_cache()
+    @functools.lru_cache(maxsize=1024)
     def accept_to_vlan(self, actions=None):
         """Get instructions to forward packet through the pipeline to
         vlan table.
@@ -66,7 +66,7 @@ class ValvePipeline(ValveManagerBase):
         """
         return self._accept_to_table(self.vlan_table, actions)
 
-    @functools.lru_cache()
+    @functools.lru_cache(maxsize=1024)
     def accept_to_classification(self, actions=None):
         """Get instructions to forward packet through the pipeline to
         classification table.
@@ -77,7 +77,7 @@ class ValvePipeline(ValveManagerBase):
         """
         return self._accept_to_table(self.classification_table, actions)
 
-    @functools.lru_cache()
+    @functools.lru_cache(maxsize=1024)
     def accept_to_l2_forwarding(self, actions=None):
         """Get instructions to forward packet through the pipeline to l2
         forwarding.
@@ -88,7 +88,7 @@ class ValvePipeline(ValveManagerBase):
         """
         return self._accept_to_table(self.output_table, actions)
 
-    @functools.lru_cache()
+    @functools.lru_cache(maxsize=1024)
     def accept_to_egress(self, actions=None):
         """Get instructions to forward packet through the pipeline to egress
         table
@@ -139,7 +139,7 @@ class ValvePipeline(ValveManagerBase):
         if self.dp.drop_broadcast_source_address:
             ofmsgs.extend(self.filter_packets(
                 {'eth_src': valve_of.mac.BROADCAST_STR}
-                ))
+            ))
 
         ofmsgs.extend(self.filter_packets(
             {'eth_type': valve_of.ECTP_ETH_TYPE}, priority_offset=10))
@@ -159,10 +159,10 @@ class ValvePipeline(ValveManagerBase):
                 vlan=vlan,
                 metadata=metadata,
                 metadata_mask=metadata_mask
-                ),
+            ),
             priority=self.dp.high_priority,
             inst=inst
-            )
+        )
 
     def add_port(self, port):
         ofmsgs = []
@@ -183,7 +183,7 @@ class ValvePipeline(ValveManagerBase):
             ofmsgs.append(self.egress_table.flowdel(self.egress_table.match(
                 metadata=port.number & mask,
                 metadata_mask=mask
-                )))
+            )))
         return ofmsgs
 
     def filter_packets(self, match_dict, priority_offset=0):
