@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 """Test FAUCET packaging"""
 
@@ -27,7 +27,7 @@ from deb_pkg_tools.deps import VersionedRelationship
 import requirements
 
 
-class CheckDebianPackageTestCase(unittest.TestCase): # pytype: disable=module-attr
+class CheckDebianPackageTestCase(unittest.TestCase):  # pytype: disable=module-attr
     """Test debian packaging."""
 
     def _parse_deb_control(self, control_file):
@@ -60,7 +60,7 @@ class CheckDebianPackageTestCase(unittest.TestCase): # pytype: disable=module-at
     def _pip_req_to_dpkg_name(self, pip_req):
         if pip_req in self.dpkg_name:
             return self.dpkg_name[pip_req]
-        return "python3-"+pip_req
+        return "python3-" + pip_req
 
     def setUp(self):
         src_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../../../')
@@ -68,7 +68,7 @@ class CheckDebianPackageTestCase(unittest.TestCase): # pytype: disable=module-at
         requirements_file = os.path.join(src_dir, 'requirements.txt')
 
         self.dpkg_name = {
-            'pyyaml': 'python3-yaml',
+            'os_ken': 'python3-os-ken',
             'prometheus_client': 'python3-prometheus-client'
         }
 
@@ -90,7 +90,7 @@ class CheckDebianPackageTestCase(unittest.TestCase): # pytype: disable=module-at
 
             if pip_req_versions:
                 debian_package_dependencies = [
-                    dpkg_name+x for x in self.faucet_dpkg_deps[dpkg_name]
+                    dpkg_name + x for x in self.faucet_dpkg_deps[dpkg_name]
                 ]
                 for pip_req_specifier, pip_req_version in pip_req_versions:
                     if pip_req_specifier == '==':
@@ -98,27 +98,27 @@ class CheckDebianPackageTestCase(unittest.TestCase): # pytype: disable=module-at
                         # versions, calculate the debian equivalent of the
                         # pip requirements match and compare that
                         lower_version = pip_req_version
-                        lower_match = '>='+lower_version
+                        lower_match = '>=' + lower_version
 
                         upper_version = pip_req_version.split('.')
                         upper_version[-1] = str(int(upper_version[-1]) + 1)
                         upper_version = '.'.join(upper_version)
-                        upper_match = '<<'+upper_version
+                        upper_match = '<<' + upper_version
 
-                        self.assertIn(dpkg_name+lower_match, debian_package_dependencies)
-                        self.assertIn(dpkg_name+upper_match, debian_package_dependencies)
+                        self.assertIn(dpkg_name + lower_match, debian_package_dependencies)
+                        self.assertIn(dpkg_name + upper_match, debian_package_dependencies)
                     elif pip_req_specifier == '<':
                         # debian/control uses << instead of <
-                        match = dpkg_name+'<<'+pip_req_version
+                        match = dpkg_name + '<<' + pip_req_version
                         self.assertIn(match, debian_package_dependencies)
                     elif pip_req_specifier == '>':
                         # debian/control uses >> instead of >
-                        match = dpkg_name+'>>'+pip_req_version
+                        match = dpkg_name + '>>' + pip_req_version
                         self.assertIn(match, debian_package_dependencies)
                     else:
-                        match = dpkg_name+pip_req_specifier+pip_req_version
+                        match = dpkg_name + pip_req_specifier + pip_req_version
                         self.assertIn(match, debian_package_dependencies)
 
 
 if __name__ == "__main__":
-    unittest.main() # pytype: disable=module-attr
+    unittest.main()  # pytype: disable=module-attr
